//
//  OnnxMLProcessorNode.hpp
//  SwitchboardOnnx
//
//  Created by Tom Dodds on 24/04/2024.
//

#pragma once

#include "ModelInstance.h"
#include <switchboard_core/SingleBusAudioProcessorNode.hpp>

#include <atomic>
#include <map>
#include <vector>

namespace switchboard::extensions::onnx {

/**
 *  OnnxMLProcessorNode class.
 *  @brief Run inference on the incoming audio using an Onnx model for audio-in audio-out use-case.
 */
class OnnxMLProcessorNode : public SingleBusAudioProcessorNode {
public:
    SB_WASM_EXPORT(OnnxMLProcessorNode)

    /**
     *  @brief Creates a OnnxMLProcessorNode instance.
     */
    SB_WASM OnnxMLProcessorNode();

    /**
     * @brief Creates a OnnxMLProcessorNode instance.
     *
     * @param config Configuration map for the node.
     */
    SB_WASM OnnxMLProcessorNode(std::map<std::string, std::any> config);

    /**
     * OnnxMLProcessorNode destructor.
     */
    SB_WASM ~OnnxMLProcessorNode();

    /**
     *  @brief Checks if the inference is enabled.
     *
     *  @returns True if the inference is enabled.
     */
    SB_WASM bool getIsEnabled() const;

    /**
     *  @brief Enables or disables the inference.
     *
     *  @param isEnabled True if the inference is enabled.
     */
    SB_WASM void setIsEnabled(const bool isEnabled);

    /**
     *  @brief Loads Model via string path
     *
     *  @param path String pointer to the model
     */
    ModelInstance* loadModel(std::string path);
    
    /**
     *  @brief Loads Model via pointer to model data
     *
     *  @param modelData Pointer to the model data
     *  @param modelSize Size of model data in bytes
     */
    SB_WASM void loadModelFromStringPointer(const char* modelData, const unsigned int modelSize);

    /**
     *  @brief A method that should be overriden to provide model data initialization
     *
     *  @param modelInstance Pointer to the ModelInstance
     */
    virtual void initialize(ModelInstance* modelInstance);
    
    /**
     *  @brief A method that should be overriden and implementation should call predict method on modelIstance
     *
     *  @param audioBufferIn Input AudioBuffer
     *  @param audioBufferOut Output AudioBuffer
     *  @param modelInstance Pointer to the ModelInstance
     */
    virtual void predict(AudioBuffer<float>& audioBufferIn, AudioBuffer<float>& audioBufferOut, ModelInstance* modelInstance);

    std::string getModelFilePath() const;
    SB_WASM void setInputFrameSize(unsigned int size);
    SB_WASM void setInputName(std::string& name);
    SB_WASM void setOutputName(std::string& name);

#pragma mark Overridden methods

    bool setBusFormat(AudioBusFormat& inputBusFormat, AudioBusFormat& outputBusFormat) override;
    bool process(AudioBus& inBus, AudioBus& outBus) override;

private:
    void createParameters();
    void setModelFilePath(const std::string modelFilePath);

    std::string modelFilePath;
    std::atomic<bool> isEnabled;
    uint sampleRate;
    uint numberOfFrames;
    std::unique_ptr<ModelInstance> modelInstance;
    std::vector<float> inputBuffer;
    std::vector<float> outputBuffer;

    uint inputFrameSize;
    std::string inputName;
    std::string outputName;
};

}
